import argparse
import asyncio
import logging
import sys
from typing import List

from unsee_dl import __version__ as unsee_dl_version
from unsee_dl.unsee import get_album_id_from_url, is_beta_album_id
from unsee_dl.unsee_beta import Client as ClientBeta
from unsee_dl.unsee_dl import Client


def main():
    asyncio.get_event_loop().run_until_complete(run_downloader())


async def download_old(album_ids: List[str], out_dir: str, group_album: bool):
    async with Client(out_path=out_dir, group_album=group_album) as client:
        for album_id in album_ids:
            # noinspection PyBroadException
            try:
                print("Downloading album {:s}...".format(album_id))
                await client.download_album(album_id)
                logging.info("Download completed for album {}.".format(album_id))
            except Exception as ex:
                logging.error(
                    "Failed downloading album {}.".format(album_id), exc_info=ex
                )


async def download_beta(album_ids: List[str], out_dir: str, group_album: bool):
    async with ClientBeta(out_path=out_dir, group_album=group_album) as client:
        await client.anonymous_login()

        for album_id in album_ids:
            # noinspection PyBroadException
            try:
                print("Downloading album {:s}...".format(album_id))
                await client.download_album(album_id)
                logging.info("Download completed for album {}.".format(album_id))
            except Exception as ex:
                logging.error(
                    "Failed downloading album {}.".format(album_id), exc_info=ex
                )


async def run_downloader():
    parser = argparse.ArgumentParser(description="unsee.cc downloader")
    parser.add_argument(
        "--version",
        action="version",
        help="Print the version",
        version=f"%(prog)s {unsee_dl_version}",
    )
    parser.add_argument(
        "-o",
        "--out",
        action="store",
        dest="out_dir",
        type=str,
        default=".",
        help="Output directory",
    )
    parser.add_argument(
        "-v",
        "--verbose",
        action="store_const",
        dest="verbose",
        default=False,
        const=True,
        help="Enable verbose output",
    )
    parser.add_argument(
        "-g",
        "--group",
        action="store_const",
        dest="group_album",
        default=False,
        const=True,
        help="Group each album in its own directory",
    )
    parser.add_argument(
        "album_ids", action="store", nargs="+", help="unsee.cc album IDs to download"
    )
    args = parser.parse_args(sys.argv[1:])

    logging.basicConfig(level=logging.DEBUG if args.verbose else logging.WARNING)

    # Download images
    album_ids = [get_album_id_from_url(url) for url in args.album_ids]
    album_ids_old_version = list(filter(lambda x: not is_beta_album_id(x), album_ids))
    album_ids_beta_version = list(filter(lambda x: is_beta_album_id(x), album_ids))

    if len(album_ids_old_version) > 0:
        await download_old(album_ids_old_version, args.out_dir, args.group_album)

    if len(album_ids_beta_version) > 0:
        await download_beta(album_ids_beta_version, args.out_dir, args.group_album)

    logging.shutdown()


if __name__ == "__main__":
    main()



import json
import logging
import ssl

import aiohttp
from aiohttp import WSMessage

from . import names
from .unsee import UnseeImage

_UNSEE_WEBSOCKET_URL = "wss://ws.unsee.cc/{}/"


class Client:
    def __init__(self, session=None, out_path=".", group_album=True):
        """
        :param session: http session
        :type session: aiohttp.ClientSession
        :param out_path: output path
        :type out_path: str
        :param group_album: should images be grouped in folders per album
        :type group_album: bool
        """
        if session:
            self.session = session
        else:
            self.session = aiohttp.ClientSession()
        self.out_path = out_path
        self.group_album = group_album
        self.token = None

    async def __aenter__(self):
        self._did_enter_with = True
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if self.session and self._did_enter_with:
            await self.session.close()

    async def download_album(self, album_id):
        unsee_name = names.get_random()
        ws_params = "?album={}&name={}".format(album_id, unsee_name)
        ssl_context = ssl.create_default_context()

        # Settings WS
        async with self.session.ws_connect(
            _UNSEE_WEBSOCKET_URL.format("settings") + ws_params, ssl=ssl_context
        ) as ws_settings:
            await ws_settings.send_str("{}")
            data = await ws_settings.receive()

            settings = json.loads(data.data, encoding="utf-8")

            logging.info("[ws_settings] {}".format(settings))

        images_info = []

        # PubSub WS
        async with self.session.ws_connect(
            _UNSEE_WEBSOCKET_URL.format("pubsub") + ws_params, ssl=ssl_context
        ) as ws_pubsub:
            data: WSMessage
            async for data in ws_pubsub:
                if data.type == aiohttp.WSMsgType.BINARY:
                    message = json.loads(data.data, encoding="utf-8")
                    logging.debug("[ws_pubsub] Received message: {}".format(message))

                    if message["type"] == "image":
                        images_info.append(message)
                        logging.info("[ws_pubsub] {}".format(message))
                    else:
                        logging.debug("[ws_pubsub] Closing socket")
                        await ws_pubsub.close()
                elif data.type == aiohttp.WSMsgType.ERROR:
                    break

        logging.info("Found {} images in album {}".format(len(images_info), album_id))
        if "title" in settings and len(settings["title"]) > 0:
            print(
                'Found album "{}" ({}) with {} images.'.format(
                    settings["title"], album_id, len(images_info)
                )
            )
        else:
            print("Found album {} with {} images.".format(album_id, len(images_info)))

        if len(images_info) <= 0:
            return

        # Imgpush WS
        async with self.session.ws_connect(
            _UNSEE_WEBSOCKET_URL.format("imgpush") + ws_params, ssl=ssl_context
        ) as ws_imgpush:
            data: WSMessage
            async for data in ws_imgpush:
                logging.debug(
                    "[ws_imgpush] received image (len: {})".format(len(data.data))
                )

                # noinspection PyBroadException
                try:
                    image = UnseeImage(
                        album_id, out_path=self.out_path, group_album=self.group_album
                    )
                    image_path = image.write_file_from_blob(data.data)
                    logging.debug("Wrote image {}".format(image_path))

                    image_info = next(
                        filter(lambda info: info["id"] == image.image_id, images_info),
                        None,
                    )
                    if image_info is not None:
                        images_info.remove(image_info)
                except:
                    logging.warning(
                        "Failed writing image from album {}".format(album_id)
                    )

                if len(images_info) <= 0:
                    await ws_imgpush.close()
